@@ -2,6 +2,7 @@ use std::any::{Any, TypeId};
 use std::collections::{BTreeMap, BTreeSet};
 use std::hash::Hasher;
 use std::time::{SystemTime, UNIX_EPOCH};
+use futures::executor::LocalPool;
 
 use ic_cdk::export::candid::utils::{ArgumentDecoder, ArgumentEncoder};
 use ic_cdk::export::candid::{decode_args, encode_args};
@@ -12,6 +13,7 @@ use crate::candid::CandidType;
 use crate::inject::{get_context, inject};
 use crate::interface::{CallResponse, Context};
 use crate::{CallHandler, Method};
+use futures::task::SpawnExt;
 
 /// A context that could be used to fake/control the behaviour of the IC when testing the canister.
 pub struct MockContext {
@@ -41,6 +43,8 @@ pub struct MockContext {
     certificate: Option<Vec<u8>>,
     /// The handlers used to handle inter-canister calls.
     handlers: Vec<Box<dyn CallHandler>>,
+    /// All of the spawned futures.
+    pool: LocalPool
 }
 
 /// A watcher can be used to inspect the calls made in a call.
@@ -99,6 +103,7 @@ impl MockContext {
             certified_data: None,
             certificate: None,
             handlers: vec![],
+            pool: LocalPool::new()
         }
     }
 
@@ -281,7 +286,7 @@ impl MockContext {
     /// Add the given handler to the handlers pipeline.
     #[inline]
     pub fn with_handler<T: 'static + CallHandler>(mut self, handler: T) -> Self {
-        self.handlers.push(Box::new(handler));
+        self.use_handler(handler);
         self
     }
 
@@ -371,6 +376,24 @@ impl MockContext {
             None => None,
         }
     }
+
+    /// Add the given handler to the call handlers pipeline.
+    #[inline]
+    pub fn use_handler<T: 'static + CallHandler>(&mut self, handler: T) {
+        self.handlers.push(Box::new(handler));
+    }
+
+    /// Remove all of the call handlers that are already registered to this context.
+    #[inline]
+    pub fn clear_handlers(&mut self) {
+        self.handlers.clear();
+    }
+
+    /// Block the current thread until all the spawned futures are complete.
+    #[inline]
+    pub fn join(&mut self) {
+        self.pool.run();
+    }
 }
 
 impl Context for MockContext {
@@ -516,10 +539,10 @@ impl Context for MockContext {
         Ok(res)
     }
 
-    fn call_raw(
+    fn call_raw<S: Into<String>>(
         &'static self,
         id: Principal,
-        method: &'static str,
+        method: S,
         args_raw: Vec<u8>,
         cycles: u64,
     ) -> CallResponse<Vec<u8>> {
@@ -530,6 +553,7 @@ impl Context for MockContext {
             );
         }
 
+        let method = method.into();
         let mut_ref = self.as_mut();
         mut_ref.balance -= cycles;
         mut_ref.is_reply_callback_mode = true;
@@ -543,8 +567,8 @@ impl Context for MockContext {
             let handler = &self.handlers[i];
             i += 1;
 
-            if handler.accept(&id, method) {
-                break handler.perform(&self.id, cycles, &id, method, &args_raw, None);
+            if handler.accept(&id, &method) {
+                break handler.perform(&self.id, cycles, &id, &method, &args_raw, None);
             }
         };
 
@@ -582,6 +606,13 @@ impl Context for MockContext {
             None => None,
         }
     }
+
+    #[inline]
+    fn spawn<F: 'static + std::future::Future<Output = ()> + std::marker::Send>(&self, future: F) {
+        // TODO(qti3e) Setup the context in the thread.
+        let spawner = self.pool.spawner();
+        spawner.spawn(future).unwrap();
+    }
 }
 
 impl Default for Watcher {
@@ -749,61 +780,53 @@ mod tests {
 
         use crate::interfaces::management::WithCanisterId;
         use crate::interfaces::*;
-        use crate::Context;
-        use crate::{get_context, Principal};
+        use crate::Principal;
+        use crate::ic;
 
         /// An update method that returns the principal id of the caller.
         pub fn whoami() -> Principal {
-            let ic = get_context();
-            ic.caller()
+            ic::caller()
         }
 
         /// An update method that returns the principal id of the canister.
         pub fn canister_id() -> Principal {
-            let ic = get_context();
-            ic.id()
+            ic::id()
         }
 
         /// An update method that returns the balance of the canister.
         pub fn balance() -> u64 {
-            let ic = get_context();
-            ic.balance()
+            ic::balance()
         }
 
         /// An update method that returns the number of cycles provided by the user in the call.
         pub fn msg_cycles_available() -> u64 {
-            let ic = get_context();
-            ic.msg_cycles_available()
+            ic::msg_cycles_available()
         }
 
         /// An update method that accepts the given number of cycles from the caller, the number of
         /// accepted cycles is returned.
         pub fn msg_cycles_accept(cycles: u64) -> u64 {
-            let ic = get_context();
-            ic.msg_cycles_accept(cycles)
+            ic::msg_cycles_accept(cycles)
         }
 
         pub type Counter = BTreeMap<u64, i64>;
 
         /// An update method that increments one to the given key, the new value is returned.
         pub fn increment(key: u64) -> i64 {
-            let ic = get_context();
-            let count = ic.get_mut::<Counter>().entry(key).or_insert(0);
+            let count = ic::get_mut::<Counter>().entry(key).or_insert(0);
             *count += 1;
             *count
         }
 
         /// An update method that decrement one from the given key. The new value is returned.
         pub fn decrement(key: u64) -> i64 {
-            let ic = get_context();
-            let count = ic.get_mut::<Counter>().entry(key).or_insert(0);
+            let count = ic::get_mut::<Counter>().entry(key).or_insert(0);
             *count -= 1;
             *count
         }
 
         pub async fn withdraw(canister_id: Principal, amount: u64) -> Result<(), String> {
-            let ic = get_context();
-            let user_balance = ic.get_mut::<u64>();
+            let user_balance = ic::get_mut::<u64>();
 
             if amount > *user_balance {
                 return Err(format!("Insufficient balance."));
@@ -812,7 +835,6 @@ mod tests {
             *user_balance -= amount;
 
             match management::DepositCycles::perform_with_payment(
-                ic,
                 Principal::management_canister(),
                 (WithCanisterId { canister_id },),
                 amount,
@@ -820,11 +842,11 @@ mod tests {
             .await
             {
                 Ok(()) => {
-                    *user_balance += ic.msg_cycles_refunded();
+                    *user_balance += ic::msg_cycles_refunded();
                     Ok(())
                 }
                 Err((code, msg)) => {
-                    assert_eq!(amount, ic.msg_cycles_refunded());
+                    assert_eq!(amount, ic::msg_cycles_refunded());
                     *user_balance += amount;
                     Err(format!(
                         "An error happened during the call: {}: {}",
@@ -835,32 +857,27 @@ mod tests {
         }
 
         pub fn user_balance() -> u64 {
-            let ic = get_context();
-            *ic.get::<u64>()
+            *ic::get::<u64>()
         }
 
         pub fn pre_upgrade() {
-            let ic = get_context();
-            let map = ic.get::<Counter>();
-            ic.stable_store((map,))
+            let map = ic::get::<Counter>();
+            ic::stable_store((map,))
                 .expect("Failed to write to stable storage");
         }
 
         pub fn post_upgrade() {
-            let ic = get_context();
-            if let Ok((map,)) = ic.stable_restore() {
-                ic.store::<Counter>(map);
+            if let Ok((map,)) = ic::stable_restore() {
+                ic::store::<Counter>(map);
             }
         }
 
         pub fn set_certified_data(data: &[u8]) {
-            let ic = get_context();
-            ic.set_certified_data(data);
+            ic::set_certified_data(data);
         }
 
         pub fn data_certificate() -> Option<Vec<u8>> {
-            let ic = get_context();
-            ic.data_certificate()
+            ic::data_certificate()
         }
     }
 
